//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : main.c
//* Object              : main application written in C
//* Creation            : Hi   11/18/2002
//* Update              : JGe  08/02/2005 : SAM7X port
//*----------------------------------------------------------------------------

#include "board.h"
#include "Emac.h"
#include "mii.h"
#include <string.h>
#include <stdio.h>

//extern unsigned int AT91F_GetTickCount(void);
extern void AT91F_DBGU_Printk(char *buffer);
extern void AT91F_DisplayIpPacket(AT91PS_IPheader pIpHeader);
extern char MsgBuffer[256];
extern int AT91F_ProcessEmacPacket(AT91PS_IPheader pHeader);
extern void LED_TurnOn(unsigned int led);
extern void LED_TurnOff(unsigned int led);

// Our Ethernet MAC address and IP address
char OurEmacAddr[6] = {0x01, 0x02, 0x03, 0x04, 0x05, 0x06};
unsigned char OurIpAddr[4]  = {10, 159, 245, 183}; // {0x0A, 0xD7, 0xF5, 0xAC}

//Buffer descriptor address must be word aligned
#define AT91C_EMAC_RX_TDLIST_BASE	0x00204000 	// take care that RxtdList[NB_RCV_BUFFERS] area is available
#define AT91C_EMAC_TX_TDLIST_BASE	(AT91C_EMAC_RX_TDLIST_BASE + NB_RX_BUFFERS * sizeof(AT91S_RxTdDescriptor))
#define AT91C_EMAC_RX_PACKET_BASE	(AT91C_EMAC_TX_TDLIST_BASE + NB_TX_BUFFERS * sizeof(AT91S_TxTdDescriptor))
#define AT91C_EMAC_TX_PACKET_BASE	(AT91C_EMAC_RX_PACKET_BASE + NB_RX_BUFFERS * ETH_RX_BUFFER_SIZE)

char *RxPacket = (char *)AT91C_EMAC_RX_PACKET_BASE;
char *TxPacket = (char *)AT91C_EMAC_TX_PACKET_BASE;
AT91PS_RxTdDescriptor RxtdList = (AT91PS_RxTdDescriptor) AT91C_EMAC_RX_TDLIST_BASE;
AT91PS_TxTdDescriptor TxtdList = (AT91PS_TxTdDescriptor)AT91C_EMAC_TX_TDLIST_BASE;

unsigned int TxBuffIndex = 0;

//  mapping
//  0x00204000 - 0x0020431F : RX descriptor list  :   800 bytes
//  0x00204320 - 0x0020432F : TX descriptor list  :    16 bytes
//  0x00204330 - 0x0020752F : RX packets          : 12800 bytes
//  0x00207530 - 0x0020762F : TX packets          :   256 bytes

// ****************************************************************************************************
// ** EMAC lowlevel functions
// ****************************************************************************************************

// Enable the MDIO bit in MAC control register
void AT91F_Enable_Mdi(AT91PS_EMAC pEmac)
{
	pEmac->EMAC_NCR |= AT91C_EMAC_MPE;	// enable management port
        pEmac->EMAC_NCFGR |= (2)<<10;	        // MDC = MCK/32
}

// Disable the MDIO bit in the MAC control register
void AT91F_Disable_Mdi(AT91PS_EMAC pEmac)
{
	pEmac->EMAC_NCR &= ~AT91C_EMAC_MPE;	// disable management port
}

// Write value to the a PHY register
// Note: MDI interface is assumed to already have been enabled.
void write_phy(AT91PS_EMAC pEmac, unsigned char phy_addr, unsigned char address, unsigned int value)
{
	pEmac->EMAC_MAN = ((AT91C_EMAC_SOF & (0x01<<30)) | (2 << 16) | (1 << 28)
		| ((phy_addr & 0x1f) << 23) | (address << 18)) | (value & 0xffff);

	/* Wait until IDLE bit in Network Status register is cleared */
	while (!(pEmac->EMAC_NSR & AT91C_EMAC_IDLE));
}

// Read value stored in a PHY register.
// Note: MDI interface is assumed to already have been enabled.
void read_phy(AT91PS_EMAC pEmac, unsigned char phy_addr, unsigned char address, unsigned int *value)
{
	pEmac->EMAC_MAN = (AT91C_EMAC_SOF & (0x01<<30)) | (2 << 16) | (2 << 28)
		| ((phy_addr & 0x1f) << 23) | (address << 18);

	/* Wait until IDLE bit in Network Status register is cleared */
	while (!(pEmac->EMAC_NSR & AT91C_EMAC_IDLE));
	*value = (pEmac->EMAC_MAN & 0x0000ffff);	
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_GetLinkSpeed
//* \brief This function configure the link speed
//*----------------------------------------------------------------------------
int AT91F_GetLinkSpeed( AT91PS_EMAC pEmac)      // \arg Pointer to AT91PS_EMAC service
{
	unsigned int bmsr, bmcr, lpa, mac_cfg;
	unsigned int speed, duplex;

	/* Link status is latched, so read twice to get current value */
	read_phy(pEmac, AT91C_PHY_ADDR, MII_BMSR, &bmsr);
	read_phy(pEmac, AT91C_PHY_ADDR, MII_BMSR, &bmsr);
	if (!(bmsr & BMSR_LSTATUS))
		 return -1;			/* no link */

	read_phy(pEmac, AT91C_PHY_ADDR, MII_BMCR, &bmcr);
	if (bmcr & BMCR_ANENABLE) {				/* AutoNegotiation is enabled */
		if (!(bmsr & BMSR_ANEGCOMPLETE))
			return -2;						/* auto-negotitation in progress */

		read_phy(pEmac, AT91C_PHY_ADDR, MII_LPA, &lpa);
		if ((lpa & LPA_100FULL) || (lpa & LPA_100HALF))
			speed = SPEED_100;
		else
			speed = SPEED_10;
		if ((lpa & LPA_100FULL) || (lpa & LPA_10FULL))
			duplex = DUPLEX_FULL;
		else duplex = DUPLEX_HALF;
	} else {
		speed = (bmcr & BMCR_SPEED100) ? SPEED_100 : SPEED_10;
		duplex = (bmcr & BMCR_FULLDPLX) ? DUPLEX_FULL : DUPLEX_HALF;
	}

	/* Update the MAC */
	mac_cfg = pEmac->EMAC_NCFGR & ~(AT91C_EMAC_SPD | AT91C_EMAC_FD);
	if (speed == SPEED_100) {
		if (duplex == DUPLEX_FULL)		/* 100 Full Duplex */
			pEmac->EMAC_NCFGR = mac_cfg | AT91C_EMAC_SPD | AT91C_EMAC_FD;
		else					/* 100 Half Duplex */
			pEmac->EMAC_NCFGR = mac_cfg | AT91C_EMAC_SPD;
	} else {
		if (duplex == DUPLEX_FULL)		/* 10 Full Duplex */
			pEmac->EMAC_NCFGR = mac_cfg | AT91C_EMAC_FD;
		else					/* 10 Half Duplex */
			pEmac->EMAC_NCFGR = mac_cfg;
	}
#if 1	
	if (duplex == DUPLEX_FULL)
		sprintf(MsgBuffer, "\n\r EMAC Init : %d Mbit/s FULL DUPLEX", speed);
	else
		sprintf(MsgBuffer, "\n\r EMAC Init : %d Mbit/s HALF DUPLEX", speed);
	AT91F_DBGU_Printk(MsgBuffer);
#endif
	return 0;
}

//Detect MAC and PHY and perform initialization
int AT91F_Ether_Probe(
		AT91PS_EMAC pEmac)     // \arg Pointer to AT91PS_EMAC service
{
	unsigned int phyid1, phyid2;
	unsigned int status;
	
	// Read the PHY ID registers
	AT91F_Enable_Mdi(pEmac);	
	read_phy(pEmac, AT91C_PHY_ADDR, MII_PHYSID1, &phyid1);
	read_phy(pEmac, AT91C_PHY_ADDR, MII_PHYSID2, &phyid2);

	/* AMD AM79C875: PHY_ID1 = 0x0022  PHY_ID2 = 0x5541
		Bits 3:0 Revision Number Four bit manufacturers revision number. 0001 stands for Rev. A, etc.
		These bits are masked.
	*/
	if (((phyid1 << 16) | (phyid2 & 0xfff0)) != MII_DM9161_ID) {
		AT91F_Disable_Mdi(pEmac);
		return -1;
	}
	status = AT91F_GetLinkSpeed(pEmac);
	AT91F_Disable_Mdi(pEmac);
	return status;
}



//*----------------------------------------------------------------------------
//* \fn    AT91F_EMACInit
//* \brief This function initialise the ethernet
//*----------------------------------------------------------------------------
int AT91F_EMACInit( // \return Status ( Success = 0)
		AT91PS_EMAC pEmac,     // \arg Pointer to AT91PS_EMAC service
		unsigned int pRxTdList,
		unsigned int pTxTdList)
{
	if (AT91F_Ether_Probe(pEmac))
		return -1;

	// the sequence write EMAC_SA1L and write EMAC_SA1H must be respected
	pEmac->EMAC_SA1L = ((int)OurEmacAddr[3] << 24) | ((int)OurEmacAddr[2] << 16) | ((int)OurEmacAddr[1] << 8) | OurEmacAddr[0];
	pEmac->EMAC_SA1H = ((int)OurEmacAddr[5] << 8) | OurEmacAddr[4];
	
	pEmac->EMAC_RBQP = pRxTdList;
	pEmac->EMAC_TBQP = pTxTdList;
	
	//Clear receive status register
	pEmac->EMAC_RSR  = (AT91C_EMAC_OVR | AT91C_EMAC_REC | AT91C_EMAC_BNA);
	pEmac->EMAC_NCFGR  |= (AT91C_EMAC_CAF | AT91C_EMAC_NBC );// | AT91C_EMAC_RBOF_OFFSET_2);

	pEmac->EMAC_NCR  |= (AT91C_EMAC_TE | AT91C_EMAC_RE | AT91C_EMAC_WESTAT);

 	return 0;
}

#define AT91C_RCV_OFFSET	0

//*----------------------------------------------------------------------------
//* \fn    AT91F_EmacEntry
//* \brief Initialise Emac to receive packets
//*----------------------------------------------------------------------------
int AT91F_EmacEntry(void)
{
	unsigned int i;
	unsigned int val;

	// Initialise RxtdList descriptor
	for (i = 0; i < NB_RX_BUFFERS; ++i) {
		val = (unsigned int)(RxPacket + (i * ETH_RX_BUFFER_SIZE));
		RxtdList[i].addr = val & 0xFFFFFFF8;
		RxtdList[i].U_Status.status = 0;
	}	
	// Set the WRAP bit at the end of the list descriptor
	RxtdList[NB_RX_BUFFERS-1].addr |= 0x02;

	// Initialise TxtdList descriptor
	for (i = 0; i < NB_TX_BUFFERS; ++i) {
		val = (unsigned int)(TxPacket + (i * ETH_TX_BUFFER_SIZE));
		TxtdList[i].addr = val & 0xFFFFFFF8;
		TxtdList[i].U_Status.status = 0;
	}	
	// Set the WRAP bit at the end of the list descriptor
	TxtdList[NB_TX_BUFFERS-1].U_Status.S_Status.Wrap = 1;

	return(AT91F_EMACInit(AT91C_BASE_EMAC, (unsigned int) RxtdList, (unsigned int) TxtdList));
}	


//*----------------------------------------------------------------------------
//* \fn    AT91F_TransmitPacket
//* \brief Send a packet
//*----------------------------------------------------------------------------
int AT91F_TransmitPacket(char *pData, unsigned int length)
{
	AT91PS_EMAC pEmac = (AT91PS_EMAC)AT91C_BASE_EMAC;
	volatile int intstatus = pEmac->EMAC_TSR;
		
	if ((TxtdList[TxBuffIndex].U_Status.S_Status.BuffUsed == 0))
	{
		TxtdList[TxBuffIndex].addr = (unsigned int)pData;
		TxtdList[TxBuffIndex].U_Status.S_Status.Length = length;
		TxtdList[TxBuffIndex].U_Status.S_Status.LastBuff = 1;
		if (TxBuffIndex == (NB_TX_BUFFERS - 1))
			TxBuffIndex = 0;
		else
			TxBuffIndex ++;
	}
	else
		return 2;
	pEmac->EMAC_NCR |= AT91C_EMAC_TSTART;
	return 0;
}


//*----------------------------------------------------------------------------
//* \fn    AT75F_EMACInt
//* \brief Initialise Emac to receive packets
//*----------------------------------------------------------------------------
void AT91F_EMAC_HANDLER (void)
{
  AT91S_IPheader IpHeader;
  AT91PS_EMAC pEmac = (AT91PS_EMAC)AT91C_BASE_EMAC;
  unsigned int intstatus;
  unsigned int i;

  intstatus = pEmac->EMAC_ISR;
	
  if (intstatus & AT91C_EMAC_RCOMP)
  {
    LED_TurnOn(LED1);
    if (AT91F_ProcessEmacPacket(&IpHeader) == AT91C_IPPACKET)
      AT91F_DisplayIpPacket(&IpHeader);
    pEmac->EMAC_RSR |= AT91C_EMAC_REC;		
    LED_TurnOff(LED1);      					
  }

  if(pEmac->EMAC_TSR & AT91C_EMAC_COMP == AT91C_EMAC_COMP)	
  {
    LED_TurnOn(LED2);
    pEmac->EMAC_TSR |= AT91C_EMAC_COMP;	
    for (i = 0; i <NB_TX_BUFFERS; i++)
    {
      if(TxtdList[i].U_Status.S_Status.BuffUsed == 1)
      {
        TxtdList[i].U_Status.S_Status.Length = 0;
        TxtdList[i].U_Status.S_Status.BuffUsed = 0;
      }
    }
    LED_TurnOff(LED2);
  }

  if(pEmac->EMAC_TSR & AT91C_EMAC_UBR == AT91C_EMAC_UBR )	
  {
    LED_TurnOn(LED3);	
    pEmac->EMAC_TSR |= AT91C_EMAC_UBR;	
    LED_TurnOff(LED3);
  } 		
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_IcmpChksum
//* \brief Process ICMP Checksum...
//*----------------------------------------------------------------------------
unsigned short AT91F_IcmpChksum(unsigned short *p, int len)
{
	int i, t;	
	for (i=0,t=0; i < len; i++, p++)
		t += SWAP16(*p);
	t = (t & 0xffff) + (t >> 16);
	return (~t);
}


//*----------------------------------------------------------------------------
//* \fn    AT91F_ProcessEmacPacket
//* \brief Process ARP and ICMP packets...
//*----------------------------------------------------------------------------
int AT91F_ProcessEmacPacket(AT91PS_IPheader pHeader)
{


	unsigned int i, icmp_len;
	char *pData;
	int status = AT91C_NO_IPPACKET;
	AT91PS_EMAC pEmac = (AT91PS_EMAC)AT91C_BASE_EMAC;
	
	AT91PS_EthHdr	pEth;
	AT91PS_ArpHdr	pArp;
	AT91PS_IPheader pIpHeader;
	AT91PS_IcmpEchoHdr pIcmpEcho;
	int process = 0;
	
	// Receive one packet		
	process = 0;	
	status = 0;
	
	
	for (i = 0; i < NB_RX_BUFFERS; ++i)
	{
		if(RxtdList[i].addr & AT91C_OWNERSHIP_BIT)
		{
			if (pEmac->EMAC_RSR & AT91C_EMAC_REC == AT91C_EMAC_REC)
				(pEmac->EMAC_RSR) |= AT91C_EMAC_REC;							
			process = 1;	
			break;				
		}
	}
		
	if (!process)
		return AT91C_NO_IPPACKET;
						
	process = i;

	// Process this packet
	pData      = (char *)(RxtdList[i].addr & 0xFFFFFFFC);
	pEth = (AT91PS_EthHdr)(pData + AT91C_RCV_OFFSET);

#if AT91C_DISPLAY_ALL_IPHEADER
			status = AT91C_IPPACKET;
#endif		

	switch (SWAP16(pEth->et_protlen))
	{
		case PROT_ARP: // ARP Packet format
			pArp = (AT91PS_ArpHdr)(pData + 14 + AT91C_RCV_OFFSET);

			if (SWAP16(pArp->ar_op) == ARP_REQUEST) {
		       // ARP REPLY operation
				pArp->ar_op =  SWAP16(ARP_REPLY);				
				
				// Fill the dest address and src address
				for (i = 0; i <6; i++) {
					// swap ethernet dest address and ethernet src address			
					pEth->et_dest[i] = pEth->et_src[i];
					pEth->et_src[i]  = OurEmacAddr[i];
					pArp->ar_tha[i]   = pArp->ar_sha[i];
					pArp->ar_sha[i] = OurEmacAddr[i];
					
				}									
				// swap sender IP address and target IP address
				for (i = 0; i<4; i++) {				
					pArp->ar_tpa[i] = pArp->ar_spa[i];
					pArp->ar_spa[i] = OurIpAddr[i];
				}	
                                //if ((pArp->ar_tpa[0]==10)&&(pArp->ar_tpa[1]==159)&&(pArp->ar_tpa[2]==245)&&(pArp->ar_tpa[3]==203))						  	
          			  	AT91F_TransmitPacket((pData + AT91C_RCV_OFFSET), 0x40);
			}			
		break; // case PROT_ARP
		
		case PROT_IP:	// IP protocol frame
			pIpHeader = (AT91PS_IPheader)(pData + 14 + AT91C_RCV_OFFSET);			
			pIcmpEcho = (AT91PS_IcmpEchoHdr)((char *)pIpHeader + 20);
			memcpy(pHeader, pIpHeader,sizeof(AT91S_IPheader));

#if AT91C_DISPLAY_ALL_IPHEADER
			status = AT91C_IPPACKET;
#endif		
			switch(pIpHeader->ip_p)
			{
				case PROT_ICMP:		
					// set the status variable to display in main.c only ICMP packets
					status = AT91C_IPPACKET;
				
					// if ICMP_ECHO_REQUEST ==> resp = ICMP_ECHO_REPLY
					if(pIcmpEcho->type == ICMP_ECHO_REQUEST)
					{
						pIcmpEcho->type = ICMP_ECHO_REPLY;
						pIcmpEcho->code = 0;
						pIcmpEcho->cksum = 0;

						// Checksum of the ICMP Message				
   						icmp_len = (SWAP16(pIpHeader->ip_len) - 20);
						if (icmp_len % 2)
						{
							*((unsigned char *)pIcmpEcho + icmp_len) = 0;
							icmp_len ++;
						}
						icmp_len = icmp_len / sizeof(unsigned short);

					    pIcmpEcho->cksum = SWAP16(AT91F_IcmpChksum((unsigned short *) pIcmpEcho, icmp_len));
					
						// Swap IP Dest address and IP Source address
						for(i = 0; i <4; i++) {
							pIpHeader->ip_dst[i] = pIpHeader->ip_src[i];
							pIpHeader->ip_src[i] = OurIpAddr[i];
						}

						// Swap Eth Dest address and Eth Source address
						for(i = 0; i <6; i++)
						{
							// swap ethernet dest address and ethernet src address
							pEth->et_dest[i] = pEth->et_src[i];
							pEth->et_src[i]  = OurEmacAddr[i];
						}
						// send the echo_reply
						//if ((pIpHeader->ip_dst[0]==10)&&(pIpHeader->ip_dst[1]==159)&&(pIpHeader->ip_dst[2]==245)&&(pIpHeader->ip_dst[3]==203))						  	
    	  				  	  AT91F_TransmitPacket((pData + AT91C_RCV_OFFSET), SWAP16(pIpHeader->ip_len) + 14 + AT91C_RCV_OFFSET);
					}
				break; // case PROT_ICMP

				default:
				break;
			}
		break; // case PROT_IP
						
		default:
		break;
	}// switch (SWAP16(*pFrameType))




	// The packet has been processed ==> release the corresponding buffers descriptors
#if 1
	RxtdList[process].addr &= ~(AT91C_OWNERSHIP_BIT);
	if(RxtdList[process].U_Status.status & AT91C_SOF)	
	{
		if(!(RxtdList[process].U_Status.status & AT91C_EOF))		
		{	
			do
			{
				if (process == (NB_RX_BUFFERS-1))
					process = 0;
				else
					process++;
				RxtdList[process].addr &= ~(AT91C_OWNERSHIP_BIT);
			}
			while(!(RxtdList[process].U_Status.status & AT91C_EOF));		
		}
	}
//	if (pEmac->EMAC_RSR & AT91C_EMAC_REC == AT91C_EMAC_REC)
//		(pEmac->EMAC_RSR) |= AT91C_EMAC_REC;							
#endif	
	return status;
}

