/* linux/include/asm-arm/arch-at91sam7xc256/at91sam7xc256_aes.h
 * 
 * Hardware definition for the aes peripheral in the ATMEL at91sam7xc256 processor
 * 
 * Generated  11/02/2005 (17:07:41) AT91 SW Application Group from AES_6149A V1.12
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef __AT91SAM7XC256_AES_H
#define __AT91SAM7XC256_AES_H

/* -------------------------------------------------------- */
/* AES ID definitions for  AT91SAM7XC256           */
/* -------------------------------------------------------- */
#ifndef AT91C_ID_AES
#define AT91C_ID_AES   	18 /**< Advanced Encryption Standard 128-bit id */
#endif /* AT91C_ID_AES */

/* -------------------------------------------------------- */
/* AES Base Address definitions for  AT91SAM7XC256   */
/* -------------------------------------------------------- */
#define AT91C_BASE_AES       	0xFFFA4000 /**< AES base address */

/* -------------------------------------------------------- */
/* PIO definition for AES hardware peripheral */
/* -------------------------------------------------------- */

/* -------------------------------------------------------- */
/* Register offset definition for AES hardware peripheral */
/* -------------------------------------------------------- */
#define AES_CR 	(0x0000) 	/**< Control Register */
#define AES_MR 	(0x0004) 	/**< Mode Register */
#define AES_IER 	(0x0010) 	/**< Interrupt Enable Register */
#define AES_IDR 	(0x0014) 	/**< Interrupt Disable Register */
#define AES_IMR 	(0x0018) 	/**< Interrupt Mask Register */
#define AES_ISR 	(0x001C) 	/**< Interrupt Status Register */
#define AES_KEYWxR 	(0x0020) 	/**< Key Word x Register */
#define AES_IDATAxR 	(0x0040) 	/**< Input Data x Register */
#define AES_ODATAxR 	(0x0050) 	/**< Output Data x Register */
#define AES_IVxR 	(0x0060) 	/**< Initialization Vector x Register */
#define AES_VR 	(0x00FC) 	/**< AES Version Register */
#define AES_RPR 	(0x0100) 	/**< Receive Pointer Register */
#define AES_RCR 	(0x0104) 	/**< Receive Counter Register */
#define AES_TPR 	(0x0108) 	/**< Transmit Pointer Register */
#define AES_TCR 	(0x010C) 	/**< Transmit Counter Register */
#define AES_RNPR 	(0x0110) 	/**< Receive Next Pointer Register */
#define AES_RNCR 	(0x0114) 	/**< Receive Next Counter Register */
#define AES_TNPR 	(0x0118) 	/**< Transmit Next Pointer Register */
#define AES_TNCR 	(0x011C) 	/**< Transmit Next Counter Register */
#define AES_PTCR 	(0x0120) 	/**< PDC Transfer Control Register */
#define AES_PTSR 	(0x0124) 	/**< PDC Transfer Status Register */

/* -------------------------------------------------------- */
/* Bitfields definition for AES hardware peripheral */
/* -------------------------------------------------------- */
/* --- Register AES_CR */
#define AT91C_AES_START       (0x1 << 0 ) /**< (AES) Starts Processing */
#define AT91C_AES_SWRST       (0x1 << 8 ) /**< (AES) Software Reset */
#define AT91C_AES_LOADSEED    (0x1 << 16) /**< (AES) Random Number Generator Seed Loading */
/* --- Register AES_MR */
#define AT91C_AES_CIPHER      (0x1 << 0 ) /**< (AES) Processing Mode */
#define AT91C_AES_PROCDLY     (0xF << 4 ) /**< (AES) Processing Delay */
#define AT91C_AES_SMOD        (0x3 << 8 ) /**< (AES) Start Mode */
#define 	AT91C_AES_SMOD_MANUAL               (0x0 <<  8) /**< (AES) Manual Mode: The START bit in register AES_CR must be set to begin encryption or decryption. */
#define 	AT91C_AES_SMOD_AUTO                 (0x1 <<  8) /**< (AES) Auto Mode: no action in AES_CR is necessary (cf datasheet). */
#define 	AT91C_AES_SMOD_PDC                  (0x2 <<  8) /**< (AES) PDC Mode (cf datasheet). */
#define AT91C_AES_OPMOD       (0x7 << 12) /**< (AES) Operation Mode */
#define 	AT91C_AES_OPMOD_ECB                  (0x0 << 12) /**< (AES) ECB Electronic CodeBook mode. */
#define 	AT91C_AES_OPMOD_CBC                  (0x1 << 12) /**< (AES) CBC Cipher Block Chaining mode. */
#define 	AT91C_AES_OPMOD_OFB                  (0x2 << 12) /**< (AES) OFB Output Feedback mode. */
#define 	AT91C_AES_OPMOD_CFB                  (0x3 << 12) /**< (AES) CFB Cipher Feedback mode. */
#define 	AT91C_AES_OPMOD_CTR                  (0x4 << 12) /**< (AES) CTR Counter mode. */
#define AT91C_AES_LOD         (0x1 << 15) /**< (AES) Last Output Data Mode */
#define AT91C_AES_CFBS        (0x7 << 16) /**< (AES) Cipher Feedback Data Size */
#define 	AT91C_AES_CFBS_128_BIT              (0x0 << 16) /**< (AES) 128-bit. */
#define 	AT91C_AES_CFBS_64_BIT               (0x1 << 16) /**< (AES) 64-bit. */
#define 	AT91C_AES_CFBS_32_BIT               (0x2 << 16) /**< (AES) 32-bit. */
#define 	AT91C_AES_CFBS_16_BIT               (0x3 << 16) /**< (AES) 16-bit. */
#define 	AT91C_AES_CFBS_8_BIT                (0x4 << 16) /**< (AES) 8-bit. */
#define AT91C_AES_CKEY        (0xF << 20) /**< (AES) Countermeasure Key */
#define AT91C_AES_CTYPE       (0x1F << 24) /**< (AES) Countermeasure Type */
#define 	AT91C_AES_CTYPE_TYPE1_EN             (0x1 << 24) /**< (AES) Countermeasure type 1 is enabled. */
#define 	AT91C_AES_CTYPE_TYPE2_EN             (0x2 << 24) /**< (AES) Countermeasure type 2 is enabled. */
#define 	AT91C_AES_CTYPE_TYPE3_EN             (0x4 << 24) /**< (AES) Countermeasure type 3 is enabled. */
#define 	AT91C_AES_CTYPE_TYPE4_EN             (0x8 << 24) /**< (AES) Countermeasure type 4 is enabled. */
#define 	AT91C_AES_CTYPE_TYPE5_EN             (0x10 << 24) /**< (AES) Countermeasure type 5 is enabled. */
/* --- Register AES_IER */
#define AT91C_AES_DATRDY      (0x1 << 0 ) /**< (AES) DATRDY */
#define AT91C_AES_ENDRX       (0x1 << 1 ) /**< (AES) PDC Read Buffer End */
#define AT91C_AES_ENDTX       (0x1 << 2 ) /**< (AES) PDC Write Buffer End */
#define AT91C_AES_RXBUFF      (0x1 << 3 ) /**< (AES) PDC Read Buffer Full */
#define AT91C_AES_TXBUFE      (0x1 << 4 ) /**< (AES) PDC Write Buffer Empty */
#define AT91C_AES_URAD        (0x1 << 8 ) /**< (AES) Unspecified Register Access Detection */
/* --- Register AES_IDR */
#define AT91C_AES_DATRDY      (0x1 << 0 ) /**< (AES) DATRDY */
#define AT91C_AES_ENDRX       (0x1 << 1 ) /**< (AES) PDC Read Buffer End */
#define AT91C_AES_ENDTX       (0x1 << 2 ) /**< (AES) PDC Write Buffer End */
#define AT91C_AES_RXBUFF      (0x1 << 3 ) /**< (AES) PDC Read Buffer Full */
#define AT91C_AES_TXBUFE      (0x1 << 4 ) /**< (AES) PDC Write Buffer Empty */
#define AT91C_AES_URAD        (0x1 << 8 ) /**< (AES) Unspecified Register Access Detection */
/* --- Register AES_IMR */
#define AT91C_AES_DATRDY      (0x1 << 0 ) /**< (AES) DATRDY */
#define AT91C_AES_ENDRX       (0x1 << 1 ) /**< (AES) PDC Read Buffer End */
#define AT91C_AES_ENDTX       (0x1 << 2 ) /**< (AES) PDC Write Buffer End */
#define AT91C_AES_RXBUFF      (0x1 << 3 ) /**< (AES) PDC Read Buffer Full */
#define AT91C_AES_TXBUFE      (0x1 << 4 ) /**< (AES) PDC Write Buffer Empty */
#define AT91C_AES_URAD        (0x1 << 8 ) /**< (AES) Unspecified Register Access Detection */
/* --- Register AES_ISR */
#define AT91C_AES_DATRDY      (0x1 << 0 ) /**< (AES) DATRDY */
#define AT91C_AES_ENDRX       (0x1 << 1 ) /**< (AES) PDC Read Buffer End */
#define AT91C_AES_ENDTX       (0x1 << 2 ) /**< (AES) PDC Write Buffer End */
#define AT91C_AES_RXBUFF      (0x1 << 3 ) /**< (AES) PDC Read Buffer Full */
#define AT91C_AES_TXBUFE      (0x1 << 4 ) /**< (AES) PDC Write Buffer Empty */
#define AT91C_AES_URAD        (0x1 << 8 ) /**< (AES) Unspecified Register Access Detection */
#define AT91C_AES_URAT        (0x7 << 12) /**< (AES) Unspecified Register Access Type Status */
#define 	AT91C_AES_URAT_IN_DAT_WRITE_DATPROC (0x0 << 12) /**< (AES) Input data register written during the data processing in PDC mode. */
#define 	AT91C_AES_URAT_OUT_DAT_READ_DATPROC (0x1 << 12) /**< (AES) Output data register read during the data processing. */
#define 	AT91C_AES_URAT_MODEREG_WRITE_DATPROC (0x2 << 12) /**< (AES) Mode register written during the data processing. */
#define 	AT91C_AES_URAT_OUT_DAT_READ_SUBKEY  (0x3 << 12) /**< (AES) Output data register read during the sub-keys generation. */
#define 	AT91C_AES_URAT_MODEREG_WRITE_SUBKEY (0x4 << 12) /**< (AES) Mode register written during the sub-keys generation. */
#define 	AT91C_AES_URAT_WO_REG_READ          (0x5 << 12) /**< (AES) Write-only register read access. */

#endif /* __AT91SAM7XC256_AES_H */
