/* linux/include/asm-arm/arch-at91sam7xc256/at91sam7xc256_tdes.h
 * 
 * Hardware definition for the tdes peripheral in the ATMEL at91sam7xc256 processor
 * 
 * Generated  11/02/2005 (17:07:41) AT91 SW Application Group from  V
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write  to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef __AT91SAM7XC256_TDES_H
#define __AT91SAM7XC256_TDES_H

/* -------------------------------------------------------- */
/* TDES ID definitions for  AT91SAM7XC256           */
/* -------------------------------------------------------- */
#ifndef AT91C_ID_TDES
#define AT91C_ID_TDES  	19 /**< Triple Data Encryption Standard id */
#endif /* AT91C_ID_TDES */

/* -------------------------------------------------------- */
/* TDES Base Address definitions for  AT91SAM7XC256   */
/* -------------------------------------------------------- */
#define AT91C_BASE_TDES      	0xFFFA8000 /**< TDES base address */

/* -------------------------------------------------------- */
/* PIO definition for TDES hardware peripheral */
/* -------------------------------------------------------- */

/* -------------------------------------------------------- */
/* Register offset definition for TDES hardware peripheral */
/* -------------------------------------------------------- */
#define TDES_CR 	(0x0000) 	/**< Control Register */
#define TDES_MR 	(0x0004) 	/**< Mode Register */
#define TDES_IER 	(0x0010) 	/**< Interrupt Enable Register */
#define TDES_IDR 	(0x0014) 	/**< Interrupt Disable Register */
#define TDES_IMR 	(0x0018) 	/**< Interrupt Mask Register */
#define TDES_ISR 	(0x001C) 	/**< Interrupt Status Register */
#define TDES_KEY1WxR 	(0x0020) 	/**< Key 1 Word x Register */
#define TDES_KEY2WxR 	(0x0028) 	/**< Key 2 Word x Register */
#define TDES_KEY3WxR 	(0x0030) 	/**< Key 3 Word x Register */
#define TDES_IDATAxR 	(0x0040) 	/**< Input Data x Register */
#define TDES_ODATAxR 	(0x0050) 	/**< Output Data x Register */
#define TDES_IVxR 	(0x0060) 	/**< Initialization Vector x Register */
#define TDES_VR 	(0x00FC) 	/**< TDES Version Register */
#define TDES_RPR 	(0x0100) 	/**< Receive Pointer Register */
#define TDES_RCR 	(0x0104) 	/**< Receive Counter Register */
#define TDES_TPR 	(0x0108) 	/**< Transmit Pointer Register */
#define TDES_TCR 	(0x010C) 	/**< Transmit Counter Register */
#define TDES_RNPR 	(0x0110) 	/**< Receive Next Pointer Register */
#define TDES_RNCR 	(0x0114) 	/**< Receive Next Counter Register */
#define TDES_TNPR 	(0x0118) 	/**< Transmit Next Pointer Register */
#define TDES_TNCR 	(0x011C) 	/**< Transmit Next Counter Register */
#define TDES_PTCR 	(0x0120) 	/**< PDC Transfer Control Register */
#define TDES_PTSR 	(0x0124) 	/**< PDC Transfer Status Register */

/* -------------------------------------------------------- */
/* Bitfields definition for TDES hardware peripheral */
/* -------------------------------------------------------- */
/* --- Register TDES_CR */
#define AT91C_TDES_START      (0x1 << 0 ) /**< (TDES) Starts Processing */
#define AT91C_TDES_SWRST      (0x1 << 8 ) /**< (TDES) Software Reset */
/* --- Register TDES_MR */
#define AT91C_TDES_CIPHER     (0x1 << 0 ) /**< (TDES) Processing Mode */
#define AT91C_TDES_TDESMOD    (0x1 << 1 ) /**< (TDES) Single or Triple DES Mode */
#define AT91C_TDES_KEYMOD     (0x1 << 4 ) /**< (TDES) Key Mode */
#define AT91C_TDES_SMOD       (0x3 << 8 ) /**< (TDES) Start Mode */
#define 	AT91C_TDES_SMOD_MANUAL               (0x0 <<  8) /**< (TDES) Manual Mode: The START bit in register TDES_CR must be set to begin encryption or decryption. */
#define 	AT91C_TDES_SMOD_AUTO                 (0x1 <<  8) /**< (TDES) Auto Mode: no action in TDES_CR is necessary (cf datasheet). */
#define 	AT91C_TDES_SMOD_PDC                  (0x2 <<  8) /**< (TDES) PDC Mode (cf datasheet). */
#define AT91C_TDES_OPMOD      (0x3 << 12) /**< (TDES) Operation Mode */
#define 	AT91C_TDES_OPMOD_ECB                  (0x0 << 12) /**< (TDES) ECB Electronic CodeBook mode. */
#define 	AT91C_TDES_OPMOD_CBC                  (0x1 << 12) /**< (TDES) CBC Cipher Block Chaining mode. */
#define 	AT91C_TDES_OPMOD_OFB                  (0x2 << 12) /**< (TDES) OFB Output Feedback mode. */
#define 	AT91C_TDES_OPMOD_CFB                  (0x3 << 12) /**< (TDES) CFB Cipher Feedback mode. */
#define AT91C_TDES_LOD        (0x1 << 15) /**< (TDES) Last Output Data Mode */
#define AT91C_TDES_CFBS       (0x3 << 16) /**< (TDES) Cipher Feedback Data Size */
#define 	AT91C_TDES_CFBS_64_BIT               (0x0 << 16) /**< (TDES) 64-bit. */
#define 	AT91C_TDES_CFBS_32_BIT               (0x1 << 16) /**< (TDES) 32-bit. */
#define 	AT91C_TDES_CFBS_16_BIT               (0x2 << 16) /**< (TDES) 16-bit. */
#define 	AT91C_TDES_CFBS_8_BIT                (0x3 << 16) /**< (TDES) 8-bit. */
/* --- Register TDES_IER */
#define AT91C_TDES_DATRDY     (0x1 << 0 ) /**< (TDES) DATRDY */
#define AT91C_TDES_ENDRX      (0x1 << 1 ) /**< (TDES) PDC Read Buffer End */
#define AT91C_TDES_ENDTX      (0x1 << 2 ) /**< (TDES) PDC Write Buffer End */
#define AT91C_TDES_RXBUFF     (0x1 << 3 ) /**< (TDES) PDC Read Buffer Full */
#define AT91C_TDES_TXBUFE     (0x1 << 4 ) /**< (TDES) PDC Write Buffer Empty */
#define AT91C_TDES_URAD       (0x1 << 8 ) /**< (TDES) Unspecified Register Access Detection */
/* --- Register TDES_IDR */
#define AT91C_TDES_DATRDY     (0x1 << 0 ) /**< (TDES) DATRDY */
#define AT91C_TDES_ENDRX      (0x1 << 1 ) /**< (TDES) PDC Read Buffer End */
#define AT91C_TDES_ENDTX      (0x1 << 2 ) /**< (TDES) PDC Write Buffer End */
#define AT91C_TDES_RXBUFF     (0x1 << 3 ) /**< (TDES) PDC Read Buffer Full */
#define AT91C_TDES_TXBUFE     (0x1 << 4 ) /**< (TDES) PDC Write Buffer Empty */
#define AT91C_TDES_URAD       (0x1 << 8 ) /**< (TDES) Unspecified Register Access Detection */
/* --- Register TDES_IMR */
#define AT91C_TDES_DATRDY     (0x1 << 0 ) /**< (TDES) DATRDY */
#define AT91C_TDES_ENDRX      (0x1 << 1 ) /**< (TDES) PDC Read Buffer End */
#define AT91C_TDES_ENDTX      (0x1 << 2 ) /**< (TDES) PDC Write Buffer End */
#define AT91C_TDES_RXBUFF     (0x1 << 3 ) /**< (TDES) PDC Read Buffer Full */
#define AT91C_TDES_TXBUFE     (0x1 << 4 ) /**< (TDES) PDC Write Buffer Empty */
#define AT91C_TDES_URAD       (0x1 << 8 ) /**< (TDES) Unspecified Register Access Detection */
/* --- Register TDES_ISR */
#define AT91C_TDES_DATRDY     (0x1 << 0 ) /**< (TDES) DATRDY */
#define AT91C_TDES_ENDRX      (0x1 << 1 ) /**< (TDES) PDC Read Buffer End */
#define AT91C_TDES_ENDTX      (0x1 << 2 ) /**< (TDES) PDC Write Buffer End */
#define AT91C_TDES_RXBUFF     (0x1 << 3 ) /**< (TDES) PDC Read Buffer Full */
#define AT91C_TDES_TXBUFE     (0x1 << 4 ) /**< (TDES) PDC Write Buffer Empty */
#define AT91C_TDES_URAD       (0x1 << 8 ) /**< (TDES) Unspecified Register Access Detection */
#define AT91C_TDES_URAT       (0x3 << 12) /**< (TDES) Unspecified Register Access Type Status */
#define 	AT91C_TDES_URAT_IN_DAT_WRITE_DATPROC (0x0 << 12) /**< (TDES) Input data register written during the data processing in PDC mode. */
#define 	AT91C_TDES_URAT_OUT_DAT_READ_DATPROC (0x1 << 12) /**< (TDES) Output data register read during the data processing. */
#define 	AT91C_TDES_URAT_MODEREG_WRITE_DATPROC (0x2 << 12) /**< (TDES) Mode register written during the data processing. */
#define 	AT91C_TDES_URAT_WO_REG_READ          (0x3 << 12) /**< (TDES) Write-only register read access. */

#endif /* __AT91SAM7XC256_TDES_H */
